(ns puppetlabs.classifier.application.permissioned.rbac
  (:require [schema.core :as sc]
            [puppetlabs.rbac.services.authz :as authz]
            [puppetlabs.classifier.application.permissioned :refer [Permissions]]))
(def rbac-object-type "node_groups")

(def rbac-actions {:edit-classification "edit_classification"
                   :edit-environment "set_environment"
                   :edit-child-rules "edit_child_rules"
                   :modify-children "modify_children"
                   :view "view"})

(defn rbac-permission
  [action id]
  {:object_type rbac-object-type, :action (get rbac-actions action), :instance (str id)})

(defn rbac-permission-str
  [action id]
  (str rbac-object-type ":" (get rbac-actions action) ":" id))

(sc/defn rbac-service-permissions :- Permissions
  [authz-service]
  (let [svc authz-service
        perm-str rbac-permission-str
        any-perm? (fn [subj perm-strs]
                    (->> (authz/are-permitted? svc subj perm-strs)
                      (some identity)
                      boolean))
        action-for-any-id? (fn [subj action ids]
                             (any-perm? subj (map (partial perm-str action) (conj ids "*"))))
        perm-fn (fn [perm-name]
                  (fn [subj id anc-ids]
                    (action-for-any-id? subj perm-name (conj anc-ids id))))]
    {:all-group-access? (fn [subj]
                          (authz/is-permitted? svc subj (perm-str :view "*")))
     :group-edit-classification? (perm-fn :edit-classification)
     :group-edit-params-and-vars? (perm-fn :edit-params-and-vars)
     :group-edit-environment? (perm-fn :edit-environment)
     :group-edit-child-rules? (perm-fn :edit-child-rules)
     :group-modify-children? (perm-fn :modify-children)
     :group-view? (perm-fn :view)
     :permitted-group-actions (fn [subj id anc-ids]
                                (let [ids (conj anc-ids id)]
                                  (into #{} (for [action (keys rbac-actions)
                                                :when (action-for-any-id? subj action ids)]
                                              action))))
     :viewable-group-ids (fn [subj all-the-ids]
                           (if (authz/is-permitted? svc subj (perm-str :view "*"))
                             (set all-the-ids)
                             (into #{} (for [id all-the-ids
                                             :when (authz/is-permitted? svc subj
                                                                        (perm-str :view id))]
                                         id))))}))
